#include "gb.h"

enum {
    MLT_REQ = 0x11,
    CHR_TRN = 0x13,
    PCT_TRN = 0x14,
    MASK_EN = 0x17,
};

typedef enum {
    MASK_DISABLED,
    MASK_FREEZE,
    MASK_COLOR_3,
    MASK_COLOR_0,
} mask_mode_t;

#define SGB_PACKET_SIZE 16

static void command_ready(GB_gameboy_t *gb)
{
    /* SGB header commands are used to send the contents of the header to the SNES CPU.
       A header command looks like this:
       Command ID: 0b1111xxx1, where xxx is the packet index. (e.g. F1 for [0x104, 0x112), F3 for [0x112, 0x120))
       Checksum: Simple one byte sum for the following content bytes
       0xE content bytes. The last command, FB, is padded with zeros, so information past the header is not sent. */
    
    
    if ((gb->sgb->command[0] & 0xF1) == 0xF1) {
        uint8_t checksum = 0;
        for (unsigned i = 2; i < 0x10; i++) {
            checksum += gb->sgb->command[i];
        }
        if (checksum != gb->sgb->command[1]) {
            GB_log(gb, "Failed checksum for SGB header command, disabling SGB features\n");
            gb->sgb->disable_commands = true;
            return;
        }
        if (gb->sgb->command[0] == 0xf9) {
            if (gb->sgb->command[0xc] != 3) { // SGB Flag
                GB_log(gb, "SGB flag is not 0x03, disabling SGB features\n");
                gb->sgb->disable_commands = true;
            }
        }
        else if (gb->sgb->command[0] == 0xfb) {
            if (gb->sgb->command[0x3] != 0x33) { // Old licensee code
                GB_log(gb, "Old licensee code is not 0x33, disabling SGB features\n");
                gb->sgb->disable_commands = true;
            }
        }
        return;
    }
    

    switch (gb->sgb->command[0] >> 3) {
        case MLT_REQ:
            gb->sgb->player_count = (uint8_t[]){1, 2, 1, 4}[gb->sgb->command[1] & 3];
            gb->sgb->current_player = gb->sgb->player_count - 1;
            break;
        case MASK_EN:
            gb->sgb->mask_mode = gb->sgb->command[1] & 3;
            break;
        case CHR_TRN:
            gb->sgb->vram_transfer_countdown = 2;
            gb->sgb->tile_transfer = true;
            gb->sgb->data_transfer = false;
            gb->sgb->tile_transfer_high = gb->sgb->command[1] & 1;
            break;
        case PCT_TRN:
            gb->sgb->vram_transfer_countdown = 2;
            gb->sgb->tile_transfer = false;
            gb->sgb->data_transfer = true;
            break;
        default:
            GB_log(gb, "Unimplemented SGB command %x: ", gb->sgb->command[0] >> 3);
            for (unsigned i = 0; i < gb->sgb->command_write_index / 8; i++) {
                GB_log(gb, "%02x ", gb->sgb->command[i]);
            }
            GB_log(gb, "\n");
            ;

    }
}

void GB_sgb_write(GB_gameboy_t *gb, uint8_t value)
{
    if (!GB_is_sgb(gb)) return;
    if (gb->sgb->disable_commands) return;
    if (gb->sgb->command_write_index >= sizeof(gb->sgb->command) * 8) return;
    
    uint16_t command_size = (gb->sgb->command[0] & 7 ?: 1) * SGB_PACKET_SIZE * 8;
    if ((gb->sgb->command[0] & 0xF1) == 0xF1) {
        command_size = SGB_PACKET_SIZE * 8;
    }
    
    switch ((value >> 4) & 3) {
        case 3:
            gb->sgb->ready_for_pulse = true;
            break;
            
        case 2: // Zero
            if (!gb->sgb->ready_for_pulse || !gb->sgb->ready_for_write) return;
            if (gb->sgb->ready_for_stop) {
                if (gb->sgb->command_write_index == command_size) {
                    command_ready(gb);
                    gb->sgb->command_write_index = 0;
                    memset(gb->sgb->command, 0, sizeof(gb->sgb->command));
                }
                gb->sgb->ready_for_pulse = false;
                gb->sgb->ready_for_write = false;
                gb->sgb->ready_for_stop = false;
            }
            else {
                gb->sgb->command_write_index++;
                gb->sgb->ready_for_pulse = false;
                if (((gb->sgb->command_write_index) & (SGB_PACKET_SIZE * 8 - 1)) == 0) {
                    gb->sgb->ready_for_stop = true;
                }
            }
            break;
        case 1: // One
            if (!gb->sgb->ready_for_pulse || !gb->sgb->ready_for_write) return;
            if (gb->sgb->ready_for_stop) {
                GB_log(gb, "Corrupt SGB command.\n");
                gb->sgb->ready_for_pulse = false;
                gb->sgb->ready_for_write = false;
                gb->sgb->command_write_index = 0;
                memset(gb->sgb->command, 0, sizeof(gb->sgb->command));
            }
            else {
                gb->sgb->command[gb->sgb->command_write_index / 8] |= 1 << (gb->sgb->command_write_index & 7);
                gb->sgb->command_write_index++;
                gb->sgb->ready_for_pulse = false;
                if (((gb->sgb->command_write_index) & (SGB_PACKET_SIZE * 8 - 1)) == 0) {
                    gb->sgb->ready_for_stop = true;
                }
            }
            break;
        
        case 0:
            if (!gb->sgb->ready_for_pulse) return;
            gb->sgb->ready_for_pulse = false;
            gb->sgb->ready_for_write = true;
            gb->sgb->ready_for_pulse = false;
            if (gb->sgb->player_count > 1 && (value & 0x30) != (gb->io_registers[GB_IO_JOYP] & 0x30)) {
                gb->sgb->current_player++;
                gb->sgb->current_player &= gb->sgb->player_count - 1;
            }
            break;
            
        default:
            break;
    }
}

static inline uint8_t scale_channel(uint8_t x)
{
    return (x << 3) | (x >> 2);
}

static uint32_t convert_rgb15(GB_gameboy_t *gb, uint16_t color)
{
    uint8_t r = (color) & 0x1F;
    uint8_t g = (color >> 5) & 0x1F;
    uint8_t b = (color >> 10) & 0x1F;
    
    r = scale_channel(r);
    g = scale_channel(g);
    b = scale_channel(b);
    
    return gb->rgb_encode_callback(gb, r, g, b);
}

static uint32_t convert_rgb15_with_fade(GB_gameboy_t *gb, uint16_t color, uint8_t fade)
{
    uint8_t r = ((color) & 0x1F) - fade;
    uint8_t g = ((color >> 5) & 0x1F) - fade;
    uint8_t b = ((color >> 10) & 0x1F) - fade;
    
    if (r >= 0x20) r = 0;
    if (g >= 0x20) g = 0;
    if (b >= 0x20) b = 0;
    
    r = scale_channel(r);
    g = scale_channel(g);
    b = scale_channel(b);
    
    return gb->rgb_encode_callback(gb, r, g, b);
}

void GB_sgb_render(GB_gameboy_t *gb)
{
    if (!gb->screen || !gb->rgb_encode_callback) return;
    
    uint32_t colors[] = {
        gb->rgb_encode_callback(gb, 0xf7, 0xe7, 0xc6),
        gb->rgb_encode_callback(gb, 0xd6, 0x8e, 0x49),
        gb->rgb_encode_callback(gb, 0xa6, 0x37, 0x25),
        gb->rgb_encode_callback(gb, 0x33, 0x1e, 0x50)
    };
    
    switch ((mask_mode_t) gb->sgb->mask_mode) {
        case MASK_DISABLED:
            memcpy(gb->sgb->effective_screen_buffer,
                   gb->sgb->screen_buffer,
                   sizeof(gb->sgb->effective_screen_buffer));
            break;
        case MASK_FREEZE:
            break;
        
        case MASK_COLOR_3:
            memset(gb->sgb->effective_screen_buffer,
                   3,
                   sizeof(gb->sgb->effective_screen_buffer));
            break;
        case MASK_COLOR_0:
            memset(gb->sgb->effective_screen_buffer,
                   0,
                   sizeof(gb->sgb->effective_screen_buffer));
    }
    
    if (gb->sgb->vram_transfer_countdown) {
        if (--gb->sgb->vram_transfer_countdown == 0) {
            if (gb->sgb->tile_transfer) {
                uint8_t *base = &gb->sgb->pending_border.tiles[gb->sgb->tile_transfer_high? 0x80 * 8 * 8 : 0];
                for (unsigned tile = 0; tile < 0x80; tile++) {
                    unsigned tile_x = (tile % 10) * 16;
                    unsigned tile_y = (tile / 10) * 8;
                    for (unsigned y = 0; y < 0x8; y++) {
                        for (unsigned x = 0; x < 0x8; x++) {
                            base[tile * 8 * 8 + y * 8 + x] = gb->sgb->screen_buffer[(tile_x + x) + (tile_y + y) * 160] +
                                                             gb->sgb->screen_buffer[(tile_x + x + 8) + (tile_y + y) * 160] * 4;
                        }
                    }
                }
                
            }
            else if (gb->sgb->data_transfer) {
                for (unsigned tile = 0; tile < 0x88; tile++) {
                    unsigned tile_x = (tile % 20) * 8;
                    unsigned tile_y = (tile / 20) * 8;
                    for (unsigned y = 0; y < 0x8; y++) {
                        static const uint16_t pixel_to_bits[4] = {0x0000, 0x0080, 0x8000, 0x8080};
                        uint16_t *data = &gb->sgb->pending_border.raw_data[tile * 8 + y];
                        *data = 0;
                        for (unsigned x = 0; x < 8; x++) {
                            *data |= pixel_to_bits[gb->sgb->screen_buffer[(tile_x + x) + (tile_y + y) * 160] & 3] >> x;
                        }
                    }
                }
                gb->sgb->border_animation = 64;
            }
        }
    }
    
    uint32_t *output = &gb->screen[48 + 40 * 256];
    uint8_t *input = gb->sgb->effective_screen_buffer;
    for (unsigned y = 0; y < 144; y++) {
        for (unsigned x = 0; x < 160; x++) {
            *(output++) = colors[*(input++) & 3];
        }
        output += 256 - 160;
    }
    
    uint32_t border_colors[16 * 4];
    if (gb->sgb->border_animation == 0) {
        for (unsigned i = 0; i < 16 * 4; i++) {
            border_colors[i] = convert_rgb15(gb, gb->sgb->border.palette[i]);
        }
    }
    else if (gb->sgb->border_animation > 32) {
        gb->sgb->border_animation--;
        for (unsigned i = 0; i < 16 * 4; i++) {
            border_colors[i] = convert_rgb15_with_fade(gb, gb->sgb->border.palette[i], 64 - gb->sgb->border_animation);
        }
    }
    else {
        gb->sgb->border_animation--;
        for (unsigned i = 0; i < 16 * 4; i++) {
            border_colors[i] = convert_rgb15_with_fade(gb, gb->sgb->border.palette[i], gb->sgb->border_animation);
        }
    }
    
    if (gb->sgb->border_animation == 32) {
        memcpy(&gb->sgb->border, &gb->sgb->pending_border, sizeof(gb->sgb->border));
    }
    
    for (unsigned tile_y = 0; tile_y < 28; tile_y++) {
        for (unsigned tile_x = 0; tile_x < 32; tile_x++) {
            bool gb_area = false;
            if (tile_x >= 6 && tile_x < 26 && tile_y >= 5 && tile_y < 23) {
                gb_area = true;
            }
            uint16_t tile = gb->sgb->border.map[tile_x + tile_y * 32];
            uint8_t flip_x = (tile & 0x4000)? 0x7 : 0;
            uint8_t flip_y = (tile & 0x8000)? 0x7 : 0;
            uint8_t palette = (tile >> 10) & 3;
            for (unsigned y = 0; y < 8; y++) {
                for (unsigned x = 0; x < 8; x++) {
                    uint8_t color = gb->sgb->border.tiles[(tile & 0xFF) * 64 + (x ^ flip_x) + (y ^ flip_y) * 8] & 0xF;
                    if (color == 0 && gb_area) continue;
                    gb->screen[tile_x * 8 + x + (tile_y * 8 + y) * 0x100] =
                    border_colors[palette * 16 + color];
                }
            }
        }
    }
}

void GB_sgb_load_default_border(GB_gameboy_t *gb)
{
    static const uint16_t palette[] = {
        0x5273, 0x14A5, 0x043C, 0x2529, 0x35AD, 0x2954, 0x739C, 0x5AD5,
        0x41A8, 0x3D44, 0x4100, 0x20E7, 0x358C, 0x3DCE, 0x4A31, 0x1C75};
    static const uint16_t tilemap[] = {
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1001, 0x1002, 0x1003, 0x1003, 0x1003, 0x1003, 0x1003,
        0x1003, 0x1003, 0x1003, 0x1003, 0x1003, 0x1003, 0x1003, 0x1003,
        0x1003, 0x1003, 0x1003, 0x1003, 0x1003, 0x1003, 0x1003, 0x1003,
        0x1003, 0x1003, 0x1003, 0x1003, 0x1003, 0x5002, 0x5001, 0x1000,
        0x1000, 0x1004, 0x1005, 0x1005, 0x1005, 0x1006, 0x1007, 0x1008,
        0x1009, 0x100A, 0x100B, 0x100C, 0x100D, 0x100E, 0x100F, 0x1010,
        0x1011, 0x1012, 0x1013, 0x1014, 0x1015, 0x1016, 0x1017, 0x1018,
        0x1019, 0x101A, 0x101B, 0x1005, 0x1005, 0x1005, 0x5004, 0x1000,
        0x1000, 0x101C, 0x101D, 0x101D, 0x101D, 0x101E, 0x101F, 0x1020,
        0x1021, 0x1022, 0x1023, 0x1024, 0x1025, 0x1026, 0x1027, 0x1028,
        0x1029, 0x102A, 0x102B, 0x102C, 0x102D, 0x102E, 0x102F, 0x1030,
        0x1031, 0x1032, 0x1033, 0x101D, 0x101D, 0x101D, 0xC01C, 0x1000,
        0x1000, 0x101C, 0x1034, 0x1034, 0x1034, 0x1035, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0xC035, 0x1034, 0x1034, 0x1034, 0xC01C, 0x1000,
        0x1000, 0x101C, 0x1034, 0x1034, 0x1034, 0x1035, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0xC035, 0x1034, 0x1034, 0x1034, 0xC01C, 0x1000,
        0x1000, 0x101C, 0x1034, 0x1034, 0x1034, 0x1035, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0xC035, 0x1034, 0x1034, 0x1034, 0xC01C, 0x1000,
        0x1000, 0x101C, 0x1034, 0x1034, 0x1034, 0x1035, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0xC035, 0x1034, 0x1034, 0x1034, 0xC01C, 0x1000,
        0x1000, 0x101C, 0x1034, 0x1034, 0x1034, 0x1035, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0xC035, 0x1034, 0x1034, 0x1034, 0xC01C, 0x1000,
        0x1000, 0x101C, 0x1034, 0x1036, 0x1034, 0x1035, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0xC035, 0x1034, 0x1034, 0x1034, 0xC01C, 0x1000,
        0x1000, 0x101C, 0x1034, 0x1037, 0x1034, 0x1035, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0xC035, 0x1034, 0x1034, 0x1034, 0xC01C, 0x1000,
        0x1000, 0x101C, 0x1034, 0x1034, 0x1034, 0x1035, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0xC035, 0x1034, 0x1034, 0x1034, 0xC01C, 0x1000,
        0x1000, 0x101C, 0x1034, 0x1034, 0x1034, 0x1035, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0xC035, 0x1034, 0x1034, 0x1034, 0xC01C, 0x1000,
        0x1000, 0x101C, 0x1034, 0x1034, 0x1034, 0x1035, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0xC035, 0x1034, 0x1034, 0x1034, 0xC01C, 0x1000,
        0x1000, 0x101C, 0x1034, 0x1034, 0x1034, 0x1035, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0xC035, 0x1034, 0x1034, 0x1034, 0xC01C, 0x1000,
        0x1000, 0x101C, 0x1034, 0x1034, 0x1034, 0x1035, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0xC035, 0x1034, 0x1034, 0x1034, 0xC01C, 0x1000,
        0x1000, 0x101C, 0x1034, 0x1034, 0x1034, 0x1035, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0xC035, 0x1034, 0x1034, 0x1034, 0xC01C, 0x1000,
        0x1000, 0x101C, 0x1034, 0x1034, 0x1034, 0x1035, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0xC035, 0x1034, 0x1034, 0x1034, 0x1038, 0x1000,
        0x1000, 0x101C, 0x1034, 0x1034, 0x1034, 0x1035, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0xC035, 0x1034, 0x1034, 0x1034, 0x1039, 0x1000,
        0x1000, 0x101C, 0x1034, 0x1034, 0x1034, 0x1035, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0xC035, 0x1034, 0x1034, 0x1034, 0x103A, 0x1000,
        0x1000, 0x101C, 0x1034, 0x1034, 0x1034, 0x1035, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0xC035, 0x1034, 0x1034, 0x103B, 0x103C, 0x1000,
        0x1000, 0x5038, 0x1034, 0x1034, 0x1034, 0x103D, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0x503D, 0x1034, 0x103E, 0x103F, 0x1000, 0x1000,
        0x1000, 0x1040, 0x1041, 0x1034, 0x1034, 0x1042, 0x1043, 0x1044,
        0x1044, 0x1044, 0x1044, 0x1044, 0x1044, 0x1044, 0x1044, 0x1044,
        0x1044, 0x1044, 0x1044, 0x1044, 0x1044, 0x1044, 0x1044, 0x1044,
        0x1044, 0x1045, 0x1046, 0x1047, 0x1048, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1049, 0x104A, 0x104B, 0x104B, 0x104B, 0x104B, 0x104B,
        0x104B, 0x104B, 0x104B, 0x104B, 0x104B, 0x104B, 0x104B, 0x104B,
        0x104B, 0x104B, 0x104B, 0x104B, 0x104B, 0x104B, 0x104B, 0x104B,
        0x104B, 0x104C, 0x104D, 0x104E, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x104F, 0x1050,
        0x1051, 0x1052, 0x1053, 0x1054, 0x1055, 0x1056, 0x1057, 0x1058,
        0x1059, 0x105A, 0x105B, 0x105C, 0x105D, 0x105E, 0x105F, 0x1060,
        0x1061, 0x1062, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1063, 0x1064,
        0x1065, 0x1066, 0x1067, 0x1068, 0x1069, 0x106A, 0x106B, 0x106C,
        0x106D, 0x106E, 0x106F, 0x1070, 0x1071, 0x1072, 0x1073, 0x1074,
        0x1075, 0x1076, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
        0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1077, 0x1078,
        0x1079, 0x107A, 0x107B, 0x107C, 0x107D, 0x107E, 0x107F, 0x1080,
        0x1081, 0x1082, 0x1083, 0x1084, 0x1085, 0x1086, 0x1087, 0x1088,
        0x1089, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000, 0x1000,
    };
    const uint8_t tiles[] = {
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x01, 0x02, 0x05, 0x01, 0x0F, 0x07,
        0x1E, 0x0E, 0x1D, 0x3D, 0x1B, 0x1B, 0x37, 0x77,
        0x00, 0x00, 0x03, 0x03, 0x06, 0x06, 0x08, 0x0B,
        0x11, 0x16, 0x22, 0x28, 0x24, 0x39, 0x48, 0x73,
        0x3F, 0xC0, 0x7F, 0x7F, 0xFF, 0xFF, 0x80, 0x80,
        0x7F, 0x7F, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0xFF, 0xFF, 0x80, 0x9F, 0x00, 0xFF, 0x7F, 0xC3,
        0x80, 0x1F, 0x00, 0x7F, 0x00, 0xFF, 0x00, 0xFF,
        0xFF, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0xFF, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0xFF, 0xFF,
        0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
        0x37, 0x37, 0x2F, 0x2F, 0x6F, 0x6F, 0x6F, 0x6F,
        0x6F, 0x6F, 0x6F, 0x6F, 0x6F, 0x6F, 0x6F, 0x6F,
        0x48, 0x2B, 0x50, 0x67, 0x10, 0x27, 0x10, 0x6F,
        0x10, 0x6F, 0x10, 0x7F, 0x10, 0x7F, 0x10, 0x7F,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0xFF,
        0x00, 0xFF, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0xFF, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
        0xFF, 0xFF, 0xFF, 0xFF, 0xF7, 0xFF, 0xF7, 0xFB,
        0xF7, 0xFB, 0xF7, 0xFB, 0xF7, 0xFB, 0xF7, 0xFB,
        0x00, 0xFF, 0x00, 0xFF, 0x0F, 0xF8, 0x0C, 0xFF,
        0x0C, 0xFF, 0x0C, 0xFF, 0x0C, 0xFF, 0x0C, 0xFF,
        0xFF, 0xFF, 0xFF, 0xFF, 0x3B, 0x3F, 0xD3, 0x53,
        0xCF, 0xAF, 0xD7, 0xAF, 0xCF, 0xAF, 0xD3, 0x53,
        0x00, 0xFF, 0x00, 0xFF, 0x47, 0x74, 0xE0, 0xB1,
        0x78, 0xE3, 0x78, 0xFF, 0x78, 0xE3, 0xE0, 0xB1,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xAF, 0x19, 0x39,
        0xF9, 0xF9, 0xE9, 0xE9, 0xF9, 0xF9, 0x19, 0x39,
        0x00, 0xFF, 0x00, 0xFF, 0xDF, 0x70, 0xA4, 0xBD,
        0x24, 0x8D, 0x34, 0xDD, 0x24, 0x8D, 0xA4, 0xBD,
        0xFF, 0xFF, 0xFF, 0xFF, 0xB7, 0xB5, 0xF5, 0xF5,
        0xF1, 0xF2, 0xF2, 0xF1, 0xF3, 0xF3, 0xF3, 0xF2,
        0x00, 0xFF, 0x00, 0xFF, 0xCE, 0x7B, 0x0C, 0xF9,
        0x0B, 0xFB, 0x0B, 0xFB, 0x09, 0xFA, 0x09, 0xFB,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xD7, 0xDE, 0xD7,
        0x8F, 0x97, 0xAF, 0xD4, 0xEF, 0x77, 0xED, 0xB3,
        0x00, 0xFF, 0x00, 0xFF, 0x38, 0xEE, 0x19, 0xCF,
        0x59, 0xDC, 0x7B, 0xFF, 0xDB, 0x9C, 0xDE, 0x7F,
        0xFF, 0xFF, 0xFF, 0xFF, 0xAF, 0xBF, 0x3D, 0x7B,
        0x9D, 0x9B, 0xDD, 0xBB, 0xFD, 0xFB, 0xDD, 0xEB,
        0x00, 0xFF, 0x00, 0xFF, 0xDF, 0x30, 0x46, 0x7F,
        0x26, 0x3F, 0x66, 0xFF, 0xE6, 0x0F, 0x36, 0xFF,
        0xFF, 0xFF, 0xFF, 0xFF, 0xBA, 0xBA, 0x9A, 0x98,
        0x9A, 0x98, 0xBA, 0xBE, 0x9A, 0x9A, 0x9A, 0x92,
        0x00, 0xFF, 0x00, 0xFF, 0xF9, 0x41, 0x43, 0xDB,
        0x43, 0xDB, 0x7D, 0xC5, 0x4D, 0xC7, 0x49, 0xD9,
        0xFF, 0xFF, 0xFF, 0xFF, 0x5C, 0x4F, 0x75, 0x71,
        0x75, 0x65, 0x7B, 0x7F, 0x6B, 0x71, 0x4A, 0x4A,
        0x00, 0xFF, 0x00, 0xFF, 0x13, 0x1F, 0x1C, 0x4C,
        0x16, 0x73, 0x0C, 0x67, 0x1A, 0x7B, 0x27, 0x45,
        0xFF, 0xFF, 0xFF, 0xFF, 0xDD, 0xDD, 0xE7, 0xE5,
        0xFD, 0xE6, 0xE7, 0xE7, 0xFD, 0xFB, 0xF7, 0xFD,
        0x00, 0xFF, 0x00, 0xFF, 0x33, 0xE6, 0x0B, 0xE6,
        0x1B, 0xFF, 0x12, 0xE1, 0x0E, 0xF7, 0x0E, 0xFB,
        0xFF, 0xFF, 0xFF, 0xFF, 0x6B, 0x65, 0x7B, 0xFD,
        0xEB, 0xDD, 0x1B, 0x2D, 0xFB, 0x3D, 0x5B, 0x5D,
        0x00, 0xFF, 0x00, 0xFF, 0x8F, 0x6E, 0x96, 0xC7,
        0xB6, 0x7F, 0x76, 0x7F, 0xE6, 0xCF, 0xC6, 0x9F,
        0xFF, 0xFF, 0xFF, 0xFF, 0xF5, 0xFB, 0xBD, 0xDB,
        0xBD, 0xDB, 0xBF, 0xDB, 0xBD, 0xDB, 0xBD, 0xDB,
        0x00, 0xFF, 0x00, 0xFF, 0xFE, 0x0F, 0x66, 0xFF,
        0x66, 0xFF, 0x67, 0xFC, 0x66, 0xFF, 0x66, 0xFF,
        0xFF, 0xFF, 0xFF, 0xFF, 0xCE, 0xCE, 0xCE, 0xCE,
        0xCE, 0xCF, 0xEF, 0xEF, 0xCF, 0xCF, 0xCE, 0xCF,
        0x00, 0xFF, 0x00, 0xFF, 0x11, 0xDF, 0x10, 0xDE,
        0x11, 0xDF, 0xF0, 0x1F, 0x10, 0xDF, 0x11, 0xDF,
        0xFF, 0xFF, 0xFF, 0xFF, 0x67, 0x6F, 0xFE, 0x4D,
        0xFE, 0x7D, 0x9E, 0x7D, 0xCE, 0xCD, 0xFE, 0x5D,
        0x00, 0xFF, 0x00, 0xFF, 0x7F, 0x18, 0xB3, 0xFF,
        0x83, 0xFF, 0xE3, 0xEF, 0x23, 0xCF, 0xB3, 0xEF,
        0xFF, 0xFF, 0xFF, 0xFF, 0xDF, 0xDE, 0xCF, 0xCF,
        0xCF, 0xCF, 0xDE, 0xDE, 0xCF, 0xCF, 0xCF, 0xCF,
        0x00, 0xFF, 0x00, 0xFF, 0xFF, 0x21, 0x20, 0xEF,
        0x20, 0xEF, 0x3F, 0xE1, 0x20, 0xEF, 0x20, 0xEF,
        0xFF, 0xFF, 0xFF, 0xFF, 0x75, 0x75, 0x34, 0x30,
        0x34, 0x30, 0x75, 0x7D, 0x34, 0x34, 0x34, 0x24,
        0x00, 0xFF, 0x00, 0xFF, 0xF3, 0x82, 0x86, 0xB6,
        0x86, 0xB6, 0xFB, 0x8A, 0x9A, 0x8E, 0x92, 0xB2,
        0xFF, 0xFF, 0xFF, 0xFF, 0xF9, 0xEB, 0xFD, 0xFF,
        0xFB, 0xF7, 0xE7, 0xEB, 0xFB, 0xF7, 0xFD, 0xFF,
        0x00, 0xFF, 0x00, 0xFF, 0xF7, 0x1E, 0x06, 0xF3,
        0x0C, 0xFF, 0xFC, 0x1F, 0x0C, 0xFF, 0x06, 0xF3,
        0xFF, 0xFF, 0xFF, 0xFF, 0xBF, 0x9F, 0xFF, 0xAE,
        0xEF, 0xCE, 0xDF, 0xDF, 0xEF, 0xCF, 0xFF, 0xAE,
        0x00, 0xFF, 0x00, 0xFF, 0xA0, 0x3E, 0x71, 0xDF,
        0x21, 0xEF, 0x30, 0xEF, 0x20, 0xEF, 0x71, 0xDF,
        0xFF, 0xFF, 0xFF, 0xFF, 0x7E, 0x6C, 0x5B, 0x4A,
        0x7B, 0xF9, 0x8D, 0x6D, 0xFB, 0xF9, 0x53, 0xD2,
        0x00, 0xFF, 0x00, 0xFF, 0x72, 0x1E, 0x37, 0x79,
        0x82, 0xFB, 0xF6, 0xFB, 0x12, 0xC3, 0xBF, 0xE9,
        0xFF, 0xFF, 0xFF, 0xFF, 0xCD, 0xC9, 0xDD, 0xF9,
        0xED, 0xF1, 0xF5, 0xE9, 0xEB, 0xF3, 0xDF, 0xFE,
        0x00, 0xFF, 0x00, 0xFF, 0xD4, 0x1D, 0x34, 0xE5,
        0x1C, 0xFD, 0x1C, 0xFD, 0x1E, 0xFD, 0x33, 0xE1,
        0xFF, 0xFF, 0xFF, 0xFF, 0xD7, 0xC5, 0xD7, 0xC7,
        0xD2, 0xC0, 0xD2, 0xC3, 0xC3, 0xC3, 0xF3, 0xB3,
        0x00, 0xFF, 0x00, 0xFF, 0x1E, 0xDB, 0x1E, 0xD8,
        0x1A, 0xDA, 0x19, 0xDB, 0x18, 0xCA, 0x68, 0xCB,
        0xFF, 0xFF, 0xFF, 0xFF, 0xF7, 0xFF, 0xF7, 0xFB,
        0xF7, 0xFB, 0xF7, 0x7B, 0xB7, 0xBB, 0x77, 0x7B,
        0x00, 0xFF, 0x00, 0xFF, 0x2F, 0x98, 0x2C, 0x9F,
        0x2C, 0x9F, 0xAC, 0x9F, 0xEC, 0x1F, 0xEC, 0x1F,
        0xFF, 0xFF, 0xFF, 0xFF, 0x3F, 0x3F, 0xDF, 0x5F,
        0xDF, 0xBF, 0xDF, 0xBF, 0xDF, 0xBF, 0xDF, 0x5F,
        0x00, 0xFF, 0x00, 0xFF, 0x40, 0x7F, 0xE0, 0xBF,
        0x60, 0xFF, 0x60, 0xFF, 0x60, 0xFF, 0xE0, 0xBF,
        0x6F, 0x6F, 0x6F, 0x6F, 0x6F, 0x6F, 0x6F, 0x6F,
        0x6F, 0x6F, 0x6F, 0x6F, 0x6F, 0x6F, 0x6F, 0x6F,
        0x10, 0x7F, 0x10, 0x7F, 0x10, 0x7F, 0x10, 0x7F,
        0x10, 0x7F, 0x10, 0x7F, 0x10, 0x7F, 0x10, 0x7F,
        0x00, 0xFF, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
        0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
        0xF7, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFE,
        0x0F, 0xF8, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
        0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFE,
        0x3B, 0x3F, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00,
        0x47, 0x74, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
        0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0x00,
        0xF9, 0xB9, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00,
        0xC4, 0x7D, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
        0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0x00,
        0xF3, 0xF3, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00,
        0x08, 0xFA, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
        0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0x00,
        0xE9, 0xF1, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00,
        0x9A, 0x39, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
        0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0x00,
        0xCD, 0xCB, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00,
        0x26, 0xC7, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
        0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0x00,
        0x98, 0x9E, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00,
        0x47, 0xDF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
        0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0x00,
        0x2E, 0x2C, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00,
        0x72, 0x5E, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
        0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0x00,
        0xFF, 0xF7, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00,
        0x0C, 0x79, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
        0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0x00,
        0xFB, 0xDD, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00,
        0x66, 0xBF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
        0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0x00,
        0xBD, 0xDB, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00,
        0x66, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
        0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0x00,
        0xCF, 0xCE, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00,
        0x11, 0xDF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
        0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0x00,
        0xEE, 0xED, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00,
        0xF3, 0x1F, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
        0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0x00,
        0xDE, 0xDF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00,
        0x3F, 0xE1, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
        0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0x00,
        0x31, 0x3D, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00,
        0x8F, 0xBE, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
        0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0x00,
        0xE9, 0xFB, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00,
        0xF7, 0x1E, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
        0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0x00,
        0xBF, 0x9E, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00,
        0xA1, 0x3F, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
        0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0x00,
        0x7E, 0x6C, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00,
        0x72, 0x1E, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
        0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0x00,
        0xCF, 0xCD, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00,
        0xD3, 0x1E, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
        0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0x00,
        0xF3, 0xD3, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00,
        0xE8, 0x3B, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
        0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0x00,
        0xB7, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00,
        0x6F, 0xD8, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
        0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0x00,
        0x3F, 0x3F, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x7F,
        0x40, 0x7F, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
        0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0x7F,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
        0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
        0xFF, 0xFE, 0xFF, 0xFE, 0xFF, 0xFE, 0xFF, 0xFE,
        0xFF, 0xFE, 0xFF, 0xFE, 0xFF, 0xFE, 0xFF, 0xFE,
        0x00, 0xFE, 0x00, 0xFE, 0x00, 0xFE, 0x00, 0xFE,
        0x00, 0xFE, 0x00, 0xFE, 0x00, 0xFE, 0x00, 0xFE,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xC3,
        0xFF, 0xFF, 0x81, 0xFF, 0x81, 0xFF, 0x00, 0xFF,
        0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xC3,
        0x3C, 0xFF, 0x00, 0x81, 0x81, 0x81, 0x00, 0x00,
        0x00, 0xFF, 0x81, 0xFF, 0xE7, 0xFF, 0xFF, 0xFF,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0x00, 0x00, 0x81, 0x81, 0x66, 0xE7, 0x3C, 0xFF,
        0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
        0xF6, 0xF6, 0xF6, 0xF6, 0xF6, 0xF6, 0xF6, 0xF6,
        0xF6, 0xF6, 0xF6, 0xF6, 0xF6, 0xF6, 0xF6, 0xF6,
        0x08, 0xFE, 0x08, 0xFE, 0x08, 0xFE, 0x08, 0xFE,
        0x08, 0xFE, 0x08, 0xFE, 0x08, 0xFE, 0x08, 0xF6,
        0xF6, 0xF6, 0xF6, 0xF6, 0xF6, 0xF6, 0xF4, 0xF4,
        0xF4, 0xF4, 0xF6, 0xF4, 0xE4, 0xE6, 0xEC, 0xEC,
        0x08, 0xF6, 0x08, 0xF4, 0x08, 0xF4, 0x0A, 0xE6,
        0x0A, 0xE4, 0x0A, 0xEE, 0x1A, 0xFE, 0x10, 0xF4,
        0xEE, 0xEE, 0xE8, 0xE8, 0xE8, 0xEC, 0xD8, 0xD8,
        0xDC, 0xDC, 0xD0, 0xD8, 0xB8, 0xB8, 0xA8, 0xA8,
        0x12, 0xCC, 0x14, 0xCC, 0x14, 0xDC, 0x20, 0xE8,
        0x24, 0x90, 0x28, 0xB8, 0x48, 0x30, 0x58, 0x30,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFE, 0xFE, 0xFE, 0xFE,
        0xFD, 0xFD, 0xF9, 0xF9, 0xFB, 0xFB, 0xF7, 0xF7,
        0x00, 0xFE, 0x00, 0xFE, 0x01, 0xFC, 0x01, 0xFC,
        0x02, 0xF9, 0x06, 0xF7, 0x04, 0xF2, 0x09, 0xE6,
        0x60, 0x60, 0x70, 0x70, 0xC0, 0xC0, 0xA0, 0xA0,
        0xC0, 0xC0, 0xC0, 0x40, 0x80, 0x80, 0x00, 0x00,
        0x80, 0xA0, 0x90, 0x40, 0x00, 0x40, 0x60, 0xC0,
        0x40, 0x80, 0xC0, 0x80, 0x80, 0x00, 0x00, 0x00,
        0xFF, 0xFE, 0xFF, 0xFE, 0xFF, 0xFE, 0xFF, 0xFE,
        0xFF, 0xFE, 0xFF, 0xFE, 0xFF, 0xFF, 0xFF, 0xFF,
        0x00, 0xFE, 0x00, 0xFE, 0x00, 0xFE, 0x00, 0xFE,
        0x00, 0xFE, 0x00, 0xFE, 0x00, 0xFF, 0x00, 0xFF,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0xFE, 0xFE, 0xFC, 0xFC, 0xFB, 0xFB, 0xE6, 0xE6,
        0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFE,
        0x01, 0xFD, 0x03, 0xF9, 0x04, 0xF1, 0x19, 0xD3,
        0xED, 0xEF, 0xCA, 0xCA, 0x98, 0x9C, 0x38, 0x38,
        0x78, 0x78, 0xF0, 0xF0, 0xE0, 0xE0, 0x40, 0xC0,
        0x13, 0xC6, 0x36, 0xBC, 0x64, 0x70, 0xC8, 0xE0,
        0x98, 0xC0, 0x30, 0x80, 0x60, 0x00, 0xC0, 0x00,
        0x6F, 0x6F, 0x6F, 0x6F, 0x2F, 0x2F, 0x37, 0x77,
        0x77, 0x77, 0x1B, 0x3B, 0x2D, 0x2D, 0x06, 0x16,
        0x10, 0x6F, 0x10, 0x27, 0x50, 0x27, 0x48, 0x6B,
        0x48, 0x13, 0x24, 0x39, 0x32, 0x08, 0x19, 0x0E,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x7F, 0x7F,
        0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
        0x00, 0xFF, 0x00, 0xFF, 0x00, 0x7F, 0x80, 0x1F,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0x00, 0xFE, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
        0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
        0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0xFF, 0x80, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
        0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
        0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0xFF, 0x00, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
        0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
        0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0xFF, 0x01, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
        0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xF0,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
        0xFF, 0xFF, 0xFF, 0xFF, 0xF8, 0xF8, 0x07, 0x07,
        0x00, 0x7F, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
        0x00, 0xFF, 0x00, 0xFC, 0x07, 0xE4, 0xF8, 0xCB,
        0xFF, 0xFF, 0xFF, 0xFF, 0xFE, 0xFE, 0xF9, 0xF9,
        0xE7, 0xE7, 0x1F, 0x1F, 0xF5, 0xF7, 0xF4, 0xDC,
        0x00, 0xFF, 0x00, 0xFE, 0x01, 0xF8, 0x06, 0xE4,
        0x18, 0x83, 0xE1, 0xBC, 0x0F, 0x78, 0x3C, 0xA0,
        0xCC, 0xCD, 0x38, 0x3E, 0x7C, 0x7C, 0xC8, 0xF8,
        0x60, 0x60, 0x40, 0xC0, 0x00, 0x00, 0x00, 0x00,
        0x33, 0xAC, 0xC6, 0x9C, 0x8C, 0x60, 0x38, 0xE0,
        0xE0, 0x80, 0xC0, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x07, 0x0B, 0x03, 0x07, 0x02, 0x03, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x0C, 0x07, 0x06, 0x00, 0x03, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x80, 0x80, 0xFF, 0xFF, 0xDF, 0x9F, 0x80, 0xFF,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x7F, 0xC3, 0x00, 0xFF, 0xE0, 0x6F, 0xFF, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0xFF,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0xFF, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0xFF, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0xFF, 0xFF, 0xFA, 0xF9, 0x01, 0xFF,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0xFF, 0x81, 0x00, 0xFF, 0x07, 0xC7, 0xFF, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0xFF, 0xFE, 0xE4, 0xEF, 0xCC, 0xFC, 0xC0, 0xC0,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x01, 0x3F, 0x1F, 0xF8, 0xFC, 0x00, 0xC0, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x90, 0xF0, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0xF0, 0x00, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01,
        0x01, 0x00, 0x03, 0x01, 0x03, 0x01, 0x03, 0x03,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01,
        0x01, 0x00, 0x03, 0x01, 0x03, 0x01, 0x03, 0x03,
        0x0F, 0x16, 0x3F, 0x1D, 0x7F, 0xF0, 0xE9, 0xE7,
        0xD3, 0xCB, 0xE7, 0x93, 0xC6, 0x02, 0x87, 0x40,
        0x1F, 0x16, 0x3F, 0x1D, 0xFF, 0xF6, 0xFF, 0xEF,
        0xFB, 0xDB, 0xF7, 0xF3, 0xC7, 0x42, 0xC7, 0xC1,
        0x00, 0x00, 0x80, 0x00, 0x80, 0x40, 0xC0, 0x00,
        0xC0, 0x00, 0x80, 0x40, 0xB0, 0x01, 0x31, 0xE9,
        0x00, 0x00, 0x80, 0x00, 0xC0, 0x40, 0xC0, 0x40,
        0xC0, 0x40, 0xC0, 0xC0, 0xB1, 0x81, 0xF9, 0xE9,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x84, 0x85, 0xC6, 0xCE, 0x87,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x84, 0x84, 0xC7, 0xC6, 0xCF, 0xC7,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0xC1, 0x80, 0xE3, 0x41,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0xC1, 0x80, 0xE3, 0xC1,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x01, 0x02, 0x03, 0xC3, 0xE3, 0xC7, 0xF7, 0x23,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x03, 0x02, 0xC3, 0xC3, 0xE7, 0xC7, 0xF7, 0x63,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x80, 0x80, 0x40, 0xF0, 0x20, 0xF0, 0xF8,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x80, 0x80, 0xC0, 0xC0, 0xF0, 0xA0, 0xF8, 0xF8,
        0x00, 0x00, 0x01, 0x00, 0x02, 0x01, 0x00, 0x07,
        0x01, 0x1E, 0x11, 0x0E, 0x00, 0x0E, 0x00, 0x1F,
        0x00, 0x00, 0x01, 0x01, 0x00, 0x07, 0x00, 0x0F,
        0x10, 0x1F, 0x11, 0x1F, 0x00, 0x1F, 0x10, 0x1F,
        0x00, 0x00, 0x00, 0x10, 0x08, 0xF4, 0x00, 0xF8,
        0x24, 0x12, 0x98, 0x64, 0x00, 0x00, 0x80, 0x40,
        0x00, 0x00, 0x10, 0xF0, 0x04, 0xFC, 0x00, 0xFC,
        0x06, 0xFE, 0xFC, 0xFC, 0x00, 0x00, 0x40, 0xC0,
        0x00, 0x00, 0x00, 0x07, 0x00, 0x07, 0x00, 0x1F,
        0x10, 0x0F, 0x00, 0x1F, 0x00, 0x7B, 0x4C, 0x33,
        0x00, 0x00, 0x07, 0x07, 0x00, 0x0F, 0x10, 0x1F,
        0x00, 0x1F, 0x00, 0x3F, 0x40, 0x7F, 0x04, 0x7F,
        0x00, 0x00, 0x00, 0x01, 0x82, 0x01, 0x00, 0x41,
        0x80, 0x41, 0x82, 0x41, 0x40, 0x83, 0x00, 0x87,
        0x00, 0x00, 0x01, 0x01, 0x82, 0x83, 0x40, 0xC3,
        0x40, 0xC3, 0x40, 0xC3, 0x40, 0xC3, 0x44, 0xC7,
        0x00, 0x00, 0x00, 0xF0, 0x10, 0xE0, 0x18, 0xE0,
        0x08, 0xF0, 0x00, 0xF0, 0x40, 0xB4, 0x49, 0xB4,
        0x00, 0x00, 0xF0, 0xF0, 0x00, 0xF0, 0x08, 0xF8,
        0x08, 0xF8, 0x08, 0xF8, 0x04, 0xFD, 0x04, 0xFD,
        0x00, 0x00, 0x00, 0x3E, 0x40, 0x3F, 0x43, 0x3C,
        0x83, 0x7C, 0x80, 0x7D, 0x20, 0xDD, 0x02, 0xDD,
        0x00, 0x00, 0x3E, 0x3E, 0x41, 0x7F, 0x01, 0x7F,
        0x81, 0xFF, 0x01, 0xFF, 0x03, 0xFD, 0x03, 0xFF,
        0x00, 0x00, 0x00, 0x3F, 0x40, 0x3F, 0x00, 0x3F,
        0x07, 0x38, 0x47, 0x30, 0x00, 0x74, 0x08, 0xF0,
        0x00, 0x00, 0x3F, 0x3F, 0x40, 0x7F, 0x00, 0x7F,
        0x00, 0x7F, 0x07, 0x7F, 0x0C, 0x7C, 0x80, 0xFF,
        0x00, 0x00, 0x00, 0xE7, 0x24, 0xD3, 0x30, 0xC7,
        0xC0, 0x1F, 0xE8, 0x17, 0x01, 0x06, 0x49, 0x06,
        0x00, 0x00, 0xE7, 0xE7, 0x10, 0xF7, 0x10, 0xF7,
        0x18, 0xFF, 0xF8, 0xFF, 0x00, 0x0F, 0x40, 0xCF,
        0x00, 0x00, 0x00, 0xF0, 0x00, 0xF8, 0x02, 0xFC,
        0xE0, 0x1F, 0x81, 0x6E, 0x83, 0x0C, 0x07, 0x18,
        0x00, 0x00, 0xF0, 0xF0, 0x00, 0xFC, 0x00, 0xFE,
        0x01, 0xFF, 0xE1, 0xFF, 0x81, 0x9F, 0x03, 0xFF,
        0x00, 0x00, 0x00, 0x01, 0x00, 0x0B, 0x18, 0x07,
        0x01, 0x3E, 0x21, 0x1C, 0x02, 0x39, 0x4C, 0x32,
        0x00, 0x00, 0x01, 0x01, 0x08, 0x0F, 0x10, 0x1F,
        0x20, 0x3F, 0x01, 0x3F, 0x03, 0x7F, 0x06, 0x7E,
        0x00, 0x00, 0x20, 0x10, 0x0C, 0xF0, 0x00, 0xFC,
        0xE1, 0x1E, 0x81, 0x6E, 0x09, 0x06, 0x00, 0x07,
        0x00, 0x00, 0x30, 0xF0, 0x04, 0xFC, 0x00, 0xFE,
        0x01, 0xFF, 0xE0, 0xFF, 0x00, 0x0F, 0x00, 0x0F,
        0x00, 0x00, 0x00, 0xF8, 0x08, 0xF0, 0x04, 0x38,
        0x00, 0x78, 0x25, 0x9A, 0x00, 0xBD, 0x90, 0x0F,
        0x00, 0x00, 0xF8, 0xF8, 0x80, 0xF8, 0x04, 0x7C,
        0x40, 0x7C, 0x83, 0xBF, 0xA0, 0xBF, 0x80, 0x9F,
        0x00, 0x00, 0x00, 0x3E, 0x00, 0x7E, 0x42, 0x38,
        0x04, 0x72, 0x18, 0xE4, 0x30, 0xC8, 0x60, 0x90,
        0x00, 0x00, 0x3E, 0x3E, 0x42, 0x7E, 0x06, 0x7E,
        0x06, 0xFE, 0x0C, 0xFC, 0x18, 0xF8, 0x30, 0xF0,
        0x03, 0x03, 0x03, 0x01, 0x01, 0x00, 0x00, 0x00,
        0x10, 0x28, 0x1C, 0x3A, 0x0F, 0x1E, 0x07, 0x0B,
        0x03, 0x03, 0x03, 0x01, 0x01, 0x00, 0x00, 0x00,
        0x38, 0x28, 0x3E, 0x3A, 0x1F, 0x1E, 0x0F, 0x0B,
        0x03, 0x00, 0x80, 0xC0, 0xC0, 0xE0, 0xF1, 0x68,
        0x79, 0x3D, 0x19, 0x2B, 0x1D, 0x03, 0xFD, 0xA1,
        0x83, 0x03, 0xC0, 0xC0, 0xE0, 0xE0, 0xF9, 0x68,
        0x7D, 0x7D, 0x3F, 0x2B, 0x1F, 0x07, 0xFD, 0xAD,
        0x63, 0x69, 0xF3, 0x4F, 0xD7, 0xC2, 0xE7, 0x9E,
        0x8F, 0xAC, 0xDE, 0x2D, 0xFE, 0x05, 0xEF, 0x95,
        0x7B, 0x69, 0xFF, 0x5F, 0xF7, 0xD2, 0xFF, 0xBF,
        0xEF, 0xAD, 0xFF, 0x6F, 0xFF, 0x47, 0xFF, 0x95,
        0xCD, 0x1C, 0x1E, 0x49, 0xBC, 0x5A, 0x34, 0xB1,
        0x79, 0x34, 0x79, 0xF3, 0xFB, 0xD7, 0xFF, 0xCE,
        0xDF, 0x5D, 0xDF, 0x4B, 0xFE, 0xDE, 0xBD, 0xB5,
        0x7D, 0x3C, 0xFB, 0xFB, 0xFF, 0xDF, 0xFF, 0xFE,
        0x67, 0xD2, 0xE7, 0x5E, 0xEF, 0x54, 0xEF, 0xDD,
        0xFF, 0x88, 0xFE, 0x98, 0xFC, 0x0A, 0xF7, 0x4C,
        0xF7, 0xD2, 0xFF, 0x5F, 0xFF, 0x55, 0xFF, 0xFD,
        0xFF, 0xAB, 0xFE, 0xFA, 0xFE, 0x5A, 0xFF, 0x5C,
        0xF3, 0x2A, 0xE7, 0x28, 0xF5, 0x4E, 0xFF, 0x80,
        0x69, 0x17, 0x15, 0xE9, 0x29, 0x51, 0xD1, 0x28,
        0xFB, 0xEA, 0xFF, 0xA9, 0xFF, 0x5F, 0xFF, 0xB2,
        0xFF, 0x77, 0xFD, 0xED, 0x79, 0x59, 0xF9, 0x38,
        0x78, 0x20, 0xF8, 0x60, 0xD0, 0xC0, 0xE0, 0x90,
        0xC8, 0xA0, 0xCC, 0x18, 0xD4, 0x18, 0xF8, 0x84,
        0xF8, 0x28, 0xF8, 0xF8, 0xF0, 0xD0, 0xF0, 0xB0,
        0xE8, 0xE0, 0xDC, 0x5C, 0xDC, 0x5C, 0xFC, 0x8C,
        0x00, 0x07, 0x02, 0x01, 0x00, 0x02, 0x00, 0x00,
        0x08, 0x00, 0x24, 0x58, 0x00, 0x7F, 0x10, 0x2F,
        0x00, 0x0F, 0x00, 0x07, 0x02, 0x03, 0x00, 0x00,
        0x08, 0x18, 0x40, 0x7F, 0x40, 0x7F, 0x20, 0x3F,
        0x20, 0xD0, 0x00, 0xF9, 0x81, 0x70, 0x0A, 0xF5,
        0x06, 0xF1, 0x80, 0x73, 0x10, 0xE7, 0x58, 0x87,
        0x10, 0xF0, 0x09, 0xF9, 0x00, 0xF9, 0x86, 0xFF,
        0x84, 0xFF, 0x08, 0xF7, 0x08, 0xFF, 0x10, 0xFF,
        0x0A, 0x75, 0x0A, 0xE1, 0x12, 0xE1, 0x00, 0xFF,
        0x00, 0xFF, 0x00, 0x81, 0xC0, 0x3F, 0x81, 0x40,
        0x0C, 0xFF, 0x08, 0xFB, 0x00, 0xFF, 0x00, 0xFF,
        0x00, 0xFF, 0x3C, 0xFF, 0x7E, 0xFF, 0xC0, 0xC1,
        0x04, 0xA3, 0x40, 0xA3, 0x44, 0xA3, 0x24, 0xC3,
        0x20, 0xC7, 0x00, 0xDF, 0x21, 0xD6, 0x21, 0xD6,
        0x24, 0xE7, 0x20, 0xE7, 0x20, 0xE7, 0x20, 0xE7,
        0x20, 0xE7, 0x18, 0xFF, 0x10, 0xFF, 0x10, 0xFF,
        0x88, 0x35, 0xC4, 0x3B, 0x04, 0x7B, 0x20, 0x5F,
        0x80, 0x5E, 0xA3, 0x1C, 0x81, 0x3C, 0x86, 0x19,
        0x04, 0xFF, 0x44, 0xFF, 0x40, 0xFF, 0x40, 0xFF,
        0xC0, 0xFF, 0xA1, 0xBF, 0xA1, 0xBF, 0x83, 0x9F,
        0x46, 0x98, 0x66, 0x98, 0xE0, 0x1A, 0x80, 0x7A,
        0x84, 0x7A, 0x44, 0xB9, 0x0C, 0x30, 0x4C, 0x30,
        0x02, 0xFE, 0x42, 0xFE, 0x42, 0xFE, 0xC2, 0xFE,
        0xC6, 0xFE, 0xC5, 0xFD, 0x04, 0x7D, 0x04, 0x7D,
        0x80, 0x7F, 0x00, 0x7F, 0x9F, 0x60, 0x80, 0x68,
        0x00, 0xE8, 0x07, 0xF8, 0x00, 0xFF, 0x00, 0xFF,
        0x80, 0xFF, 0x00, 0xFF, 0x0F, 0xFF, 0x08, 0xF8,
        0x18, 0xF8, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
        0x08, 0xA7, 0x00, 0xAF, 0xC1, 0x3E, 0x13, 0x0C,
        0x03, 0x0C, 0xD1, 0x0E, 0x10, 0xAF, 0x40, 0xBF,
        0x20, 0xEF, 0x20, 0xEF, 0xF1, 0xFF, 0x11, 0x1F,
        0x01, 0x1F, 0x40, 0xDF, 0x60, 0xFF, 0x60, 0xFF,
        0x0E, 0xF0, 0x00, 0xF8, 0xC2, 0x1C, 0x02, 0x1D,
        0x20, 0x1D, 0xC2, 0x3D, 0x02, 0xF8, 0x3C, 0xC2,
        0x06, 0xFE, 0x00, 0xFC, 0xC2, 0xFE, 0x01, 0x1F,
        0x21, 0x3F, 0x03, 0xFF, 0x02, 0xFE, 0x0E, 0xFE,
        0x4C, 0x30, 0x00, 0xF4, 0x00, 0xF4, 0x08, 0x74,
        0x40, 0x38, 0x01, 0x7E, 0x00, 0x1F, 0x18, 0x07,
        0x04, 0x7C, 0x84, 0xFC, 0x84, 0xFC, 0x04, 0x7C,
        0x00, 0x7C, 0x40, 0x7F, 0x00, 0x3F, 0x10, 0x1F,
        0x00, 0x07, 0x09, 0x06, 0x01, 0x1E, 0x00, 0x0E,
        0x03, 0x5C, 0x82, 0x7D, 0x0E, 0xF0, 0x2C, 0xC0,
        0x00, 0x0F, 0x00, 0x0F, 0x10, 0x1F, 0x01, 0x1F,
        0x41, 0x7F, 0x03, 0xFF, 0x06, 0xFE, 0x0C, 0xFC,
        0x90, 0x0F, 0x80, 0x0F, 0x89, 0x06, 0x00, 0x8E,
        0x00, 0x9E, 0x11, 0x0E, 0x01, 0x0E, 0x13, 0x0C,
        0x90, 0x9F, 0x80, 0x8F, 0x80, 0x8F, 0x80, 0x8F,
        0x91, 0x9F, 0x11, 0x1F, 0x01, 0x1F, 0x01, 0x1F,
        0x60, 0x80, 0x40, 0x00, 0x80, 0x40, 0x00, 0x80,
        0x00, 0x80, 0x00, 0x80, 0x00, 0x00, 0x00, 0x00,
        0x60, 0xE0, 0xC0, 0x40, 0xC0, 0xC0, 0x80, 0x80,
        0x80, 0x80, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x03, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0xF8, 0x05, 0x00, 0xF0, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0xFD, 0x3D, 0xF0, 0xF0, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x67, 0x10, 0x01, 0x63, 0x01, 0x03, 0x03, 0x01,
        0x03, 0x00, 0x03, 0x00, 0x00, 0x01, 0x00, 0x00,
        0xF7, 0x74, 0x63, 0x63, 0x03, 0x03, 0x03, 0x01,
        0x03, 0x00, 0x03, 0x01, 0x01, 0x01, 0x00, 0x00,
        0xE7, 0x88, 0xC0, 0xA3, 0xC0, 0x00, 0x80, 0x40,
        0x80, 0x00, 0x00, 0x80, 0x00, 0x80, 0x00, 0x00,
        0xEF, 0xA8, 0xE3, 0xE3, 0xC0, 0x40, 0xC0, 0xC0,
        0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x00, 0x00,
        0xA1, 0x54, 0x00, 0xC1, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0xF7, 0xF5, 0xC1, 0xC1, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x40, 0xA0, 0x00, 0x80, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0xE0, 0xE0, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0xD0, 0x28, 0x00, 0x70, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0xF8, 0xB8, 0x70, 0x70, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x08, 0x10, 0x00, 0x03, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x18, 0x1F, 0x03, 0x03, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x67, 0x10, 0x00, 0xC0, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0xF7, 0x77, 0xC0, 0xC0, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x80, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0xF3, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0xF7, 0xF7, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x0E, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x8E, 0x8E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x38, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x3C, 0x3C, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x7F, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0xCF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0xCF, 0xCF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0xF0, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0xF8, 0xF8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x04, 0x08, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x0E, 0x0F, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x70, 0x08, 0x00, 0xC0, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0xF8, 0xF8, 0xC0, 0xC0, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x0F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x0F, 0x0F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
    };
    
    memcpy(gb->sgb->border.map, tilemap, sizeof(tilemap));
    memcpy(gb->sgb->border.palette, palette, sizeof(palette));
    
    
    /* Expend tileset */
    for (unsigned tile = 0; tile < sizeof(tiles) / 32; tile++) {
        for (unsigned y = 0; y < 8; y++) {
            for (unsigned x = 0; x < 8; x++) {
                gb->sgb->border.tiles[tile * 8 * 8 + y * 8 + x] =
                    (tiles[tile * 32 + y * 2 +  0] & (1 << (7 ^ x)) ? 1 : 0) |
                    (tiles[tile * 32 + y * 2 +  1] & (1 << (7 ^ x)) ? 2 : 0) |
                    (tiles[tile * 32 + y * 2 + 16] & (1 << (7 ^ x)) ? 4 : 0) |
                    (tiles[tile * 32 + y * 2 + 17] & (1 << (7 ^ x)) ? 8 : 0);
            }
        }
    }
}
